@@ -203,7 +203,7 @@ NvBool nvDpyIsHdmiEvo(const NVDpyEvoRec *pDpyEvo)
 /*!
  * Updates the display's HDMI 2.0 capabilities to the RM.
  */
-void nvUpdateHdmiCaps(NVDpyEvoPtr pDpyEvo)
+void nvSendHdmiCapsToRm(NVDpyEvoPtr pDpyEvo)
 {
     NV0073_CTRL_SPECIFIC_SET_HDMI_SINK_CAPS_PARAMS params = { 0 };
     NVParsedEdidEvoPtr pParsedEdid = &pDpyEvo->parsedEdid;
@@ -221,7 +221,7 @@ void nvUpdateHdmiCaps(NVDpyEvoPtr pDpyEvo)
     params.caps = 0;
 
     /*
-     * nvUpdateHdmiCaps() gets called on dpy's connect/disconnect events
+     * nvSendHdmiCapsToRm() gets called on dpy's connect/disconnect events
      * to set/clear capabilities, clear capabilities if parsed edid
      * is not valid.
      */
@@ -691,6 +691,10 @@ static void SetHdmiAudioMute(const NVDispEvoRec *pDispEvo,
 static void EnableHdmiAudio(const NVDispEvoRec *pDispEvo,
                             const NvU32 head, const NvBool enable)
 {
+    /*
+     * XXX Is it correct to use pktType_GeneralControl to mute/unmute
+     * the audio? pktType_GeneralControl controls both the audio and video data.
+     */
     static const NvU8 InfoframeMutePacket[] = {
         pktType_GeneralControl, 0, 0, HDMI_GENCTRL_PACKET_MUTE_ENABLE, 0, 0, 0, 0,
         0, 0
@@ -998,12 +1002,13 @@ void nvHdmiDpConstructHeadAudioState(const NvU32 displayId,
         return;
     }
 
+    pAudioState->isAudioOverHdmi = nvDpyIsHdmiEvo(pDpyEvo);
+
     if (FillELDBuffer(displayId,
                       nvConnectorUsesDPLib(pDpyEvo->pConnectorEvo),
                       &pDpyEvo->parsedEdid,
                       &pAudioState->eld,
                       &pAudioState->maxFreqSupported)) {
-        pAudioState->isAudioOverHdmi = nvDpyIsHdmiEvo(pDpyEvo);
         pAudioState->enabled = TRUE;
     }
 }
@@ -1197,37 +1202,25 @@ void nvHdmiDpEnableDisableAudio(const NVDispEvoRec *pDispEvo,
         return;
     }
 
-    if (!pHeadState->audio.enabled) {
-
-        if (enable) {
-            /* Make sure to remove corresponding audio device */
+    if (!enable) {
+        /*
+         * This is pre modeset code path. If audio device is enabled
+         * (pHeadState->audio.enabled == TRUE) then invalidate ELD buffer
+         * before disabling audio.
+         */
+        if (pHeadState->audio.enabled) {
             RmSetELDAudioCaps(pDispEvo,
                               pConnectorEvo,
-                              nvDpyIdToNvU32(pConnectorEvo->displayId),
+                              pHeadState->activeRmId,
                               deviceEntry,
                               0 /* maxFreqSupported */,
                               NULL /* pEld */,
-                              NV_ELD_POWER_ON_RESET);
-        } else {
-            /* Do nothing. The audio device is already in the disabled state. */
+                              NV_ELD_PRE_MODESET);
+
+            if (nvConnectorUsesDPLib(pConnectorEvo)) {
+                SetDpAudioEnable(pDispEvo, head, FALSE /* enable */);
+            }
         }
-
-        return;
-    }
-
-    /* Invalidate ELD buffer before disabling audio */
-    if (!enable) {
-        RmSetELDAudioCaps(pDispEvo,
-                          pConnectorEvo,
-                          pHeadState->activeRmId,
-                          deviceEntry,
-                          0 /* maxFreqSupported */,
-                          NULL /* pEld */,
-                          NV_ELD_PRE_MODESET);
-    }
-
-    if (nvConnectorUsesDPLib(pConnectorEvo)) {
-        SetDpAudioEnable(pDispEvo, head, enable);
     }
 
     if (pHeadState->audio.isAudioOverHdmi) {
@@ -1236,15 +1229,34 @@ void nvHdmiDpEnableDisableAudio(const NVDispEvoRec *pDispEvo,
         SendHdmiGcp(pDispEvo, head, !enable /* avmute */);
     }
 
-    /* Populate ELD buffer after enabling audio */
     if (enable) {
-        RmSetELDAudioCaps(pDispEvo,
-                          pConnectorEvo,
-                          pHeadState->activeRmId,
-                          deviceEntry,
-                          pHeadState->audio.maxFreqSupported,
-                          &pHeadState->audio.eld,
-                          NV_ELD_POST_MODESET);
+        /*
+         * This is post modeset code path. If audio device is enabled
+         * (pHeadState->audio.enabled == TRUE) then populate ELD buffer after
+         * enabling audio, otherwise make sure to remove corresponding audio
+         * device.
+         */
+        if (pHeadState->audio.enabled) {
+            if (nvConnectorUsesDPLib(pConnectorEvo)) {
+                SetDpAudioEnable(pDispEvo, head, TRUE /* enable */);
+            }
+
+            RmSetELDAudioCaps(pDispEvo,
+                              pConnectorEvo,
+                              pHeadState->activeRmId,
+                              deviceEntry,
+                              pHeadState->audio.maxFreqSupported,
+                              &pHeadState->audio.eld,
+                              NV_ELD_POST_MODESET);
+        } else {
+            RmSetELDAudioCaps(pDispEvo,
+                              pConnectorEvo,
+                              nvDpyIdToNvU32(pConnectorEvo->displayId),
+                              deviceEntry,
+                              0 /* maxFreqSupported */,
+                              NULL /* pEld */,
+                              NV_ELD_POWER_ON_RESET);
+        }
     }
 }
 
@@ -1267,6 +1279,32 @@ void nvDpyUpdateHdmiPreModesetEvo(NVDpyEvoPtr pDpyEvo)
  */
 void nvDpyUpdateHdmiVRRCaps(NVDpyEvoPtr pDpyEvo)
 {
+
+    const NVParsedEdidEvoRec *pParsedEdid = &pDpyEvo->parsedEdid;
+    const NVDispEvoRec *pDispEvo = pDpyEvo->pDispEvo;
+    const NVDevEvoRec *pDevEvo = pDispEvo->pDevEvo;
+
+    const NvBool gpuSupportsHDMIVRR = pDevEvo->hal->caps.supportsHDMIVRR;
+
+    const NvBool dispSupportsVrr = nvDispSupportsVrr(pDispEvo);
+
+    const NvU32 edidVrrMin = pParsedEdid->info.hdmiForumInfo.vrr_min;
+
+    nvAssert(pParsedEdid->valid);
+
+    if (dispSupportsVrr && gpuSupportsHDMIVRR && (edidVrrMin > 0)) {
+        if (nvDpyIsAdaptiveSyncDefaultlisted(pDpyEvo)) {
+            pDpyEvo->vrr.type =
+                NVKMS_DPY_VRR_TYPE_ADAPTIVE_SYNC_DEFAULTLISTED;
+        } else {
+            pDpyEvo->vrr.type =
+                NVKMS_DPY_VRR_TYPE_ADAPTIVE_SYNC_NON_DEFAULTLISTED;
+        }
+
+        pDpyEvo->vrr.needsSwFramePacing = TRUE;
+
+        pDpyEvo->vrr.edidTimeoutMicroseconds = 1000000 / edidVrrMin;
+    }
 }
 
 void nvRemoveUnusedHdmiDpAudioDevice(const NVDispEvoRec *pDispEvo)
@@ -1279,7 +1317,7 @@ void nvRemoveUnusedHdmiDpAudioDevice(const NVDispEvoRec *pDispEvo)
 
         // Only connectors with assigned SORs can have audio.
         if (pConnectorEvo->or.type != NV0073_CTRL_SPECIFIC_OR_TYPE_SOR ||
-            pConnectorEvo->or.mask == 0x0) {
+            pConnectorEvo->or.primary == NV_INVALID_OR) {
             continue;
         }
 
@@ -1289,7 +1327,7 @@ void nvRemoveUnusedHdmiDpAudioDevice(const NVDispEvoRec *pDispEvo)
         // NV0073_CTRL_CMD_DFP_SET_ELD_AUDIO_CAPS takes a displayId rather than
         // an SOR index. See bug 1953489.
         if (nvIsConnectorActiveEvo(pConnectorEvo) &&
-            (pConnectorEvo->or.mask & activeSorMask) != 0x0) {
+            (NVBIT(pConnectorEvo->or.primary) & activeSorMask) != 0x0) {
             continue;
         }
 
@@ -1699,6 +1737,7 @@ void nvHdmiSetVRR(NVDispEvoPtr pDispEvo, NvU32 head, NvBool enable)
                  sizeof(empCtrl));
 
     empCtrl.EnableVRR = enable;
+    empCtrl.EnableQMS = FALSE;
 
     status = NvTiming_ConstructExtendedMetadataPacketInfoframe(&empCtrl,
                                                                &empInfoFrame);
@@ -2002,8 +2041,8 @@ NvBool nvHdmiDpySupportsFrl(const NVDpyEvoRec *pDpyEvo)
  *
  * Returns TRUE if FRL is needed, or FALSE otherwise.
  * */
-static NvBool HdmiTimingsNeedFrl(const NVDpyEvoRec *pDpyEvo,
-                                 const NVHwModeTimingsEvo *pHwTimings)
+NvBool nvHdmiTimingsNeedFrl(const NVDpyEvoRec *pDpyEvo,
+                            const NVHwModeTimingsEvo *pHwTimings)
 {
     const NvU32 pixelClock = (pHwTimings->yuv420Mode == NV_YUV420_MODE_HW) ?
         (pHwTimings->pixelClock / 2) : pHwTimings->pixelClock;
@@ -2017,11 +2056,47 @@ static NvBool HdmiTimingsNeedFrl(const NVDpyEvoRec *pDpyEvo,
     return pixelClock > pDpyEvo->maxSingleLinkPixelClockKHz;
 }
 
+static NvU64 GetHdmiFrlLinkRate(HDMI_FRL_DATA_RATE frlRate)
+{
+    const NvU64 giga = 1000000000ULL;
+    NvU64 hdmiLinkRate = 0;
+    switch(frlRate )
+    {
+        case HDMI_FRL_DATA_RATE_NONE:
+            hdmiLinkRate = 0;
+            break;
+        case HDMI_FRL_DATA_RATE_3LANES_3GBPS:
+            hdmiLinkRate =  3 * giga;
+            break;
+        case HDMI_FRL_DATA_RATE_3LANES_6GBPS:
+        case HDMI_FRL_DATA_RATE_4LANES_6GBPS:
+            hdmiLinkRate =  6 * giga;
+            break;
+        case HDMI_FRL_DATA_RATE_4LANES_8GBPS:
+            hdmiLinkRate =  8 * giga;
+            break;
+        case HDMI_FRL_DATA_RATE_4LANES_10GBPS:
+            hdmiLinkRate = 10 * giga;
+            break;
+        case HDMI_FRL_DATA_RATE_4LANES_12GBPS:
+            hdmiLinkRate = 12 * giga;
+            break;
+        case HDMI_FRL_DATA_RATE_UNSPECIFIED:
+            nvAssert(!"Unspecified FRL data rate");
+            break;
+    };
+
+    return hdmiLinkRate;
+}
+
 NvBool nvHdmiFrlQueryConfig(
     const NVDpyEvoRec *pDpyEvo,
     const NvModeTimings *pModeTimings,
-    NVHwModeTimingsEvo *pHwTimings,
-    const struct NvKmsModeValidationParams *pValidationParams)
+    const NVHwModeTimingsEvo *pHwTimings,
+    const NvBool b2Heads1Or,
+    const struct NvKmsModeValidationParams *pValidationParams,
+    HDMI_FRL_CONFIG *pConfig,
+    NVDscInfoEvoRec *pDscInfo)
 {
     const NVDispEvoRec *pDispEvo = pDpyEvo->pDispEvo;
     const NVDevEvoRec *pDevEvo = pDispEvo->pDevEvo;
@@ -2031,14 +2106,15 @@ NvBool nvHdmiFrlQueryConfig(
     NVT_TIMING nvtTiming = { };
     NVHDMIPKT_RESULT ret;
 
-    if (!nvDpyIsHdmiEvo(pDpyEvo) ||
-        !HdmiTimingsNeedFrl(pDpyEvo, pHwTimings)) {
+    if (pHwTimings->protocol != NVKMS_PROTOCOL_SOR_HDMI_FRL) {
+        nvkms_memset(pDscInfo, 0, sizeof(*pDscInfo));
+        nvkms_memset(pConfig, 0, sizeof(*pConfig));
         return TRUE;
     }
 
-    if (!nvHdmiDpySupportsFrl(pDpyEvo)) {
-        return FALSE;
-    }
+    nvAssert(nvDpyIsHdmiEvo(pDpyEvo) &&
+                nvHdmiDpySupportsFrl(pDpyEvo) &&
+                nvHdmiTimingsNeedFrl(pDpyEvo, pHwTimings));
 
     /* See if we can find an NVT_TIMING for this mode from the EDID. */
     pNvtTiming = nvFindEdidNVT_TIMING(pDpyEvo, pModeTimings, pValidationParams);
@@ -2102,14 +2178,23 @@ NvBool nvHdmiFrlQueryConfig(
             videoTransportInfo.packing = HDMI_PIXEL_PACKING_YCbCr420;
             break;
     }
-    /* TODO: implement 2head1or+FRL */
-    videoTransportInfo.bDualHeadMode = FALSE;
+
+    videoTransportInfo.bDualHeadMode = b2Heads1Or;
 
     clientControl.option = HDMI_QUERY_FRL_HIGHEST_PIXEL_QUALITY;
 
     if (pValidationParams->forceDsc) {
         clientControl.enableDSC = TRUE;
     }
+
+    /*
+     * 2Heads1Or requires either YUV420 or DSC; if b2Heads1Or is enabled
+     * but YUV420 is not, force DSC.
+     */
+    if (b2Heads1Or && (pHwTimings->yuv420Mode != NV_YUV420_MODE_HW)) {
+        clientControl.enableDSC = TRUE;
+    }
+
     if (pValidationParams->dscOverrideBitsPerPixelX16 != 0) {
         clientControl.forceBppx16 = TRUE;
         clientControl.bitsPerPixelX16 =
@@ -2121,7 +2206,54 @@ NvBool nvHdmiFrlQueryConfig(
                                 &clientControl,
                                 &pDpyEvo->hdmi.srcCaps,
                                 &pDpyEvo->hdmi.sinkCaps,
-                                &pHwTimings->hdmiFrlConfig);
+                                pConfig);
+
+    if ((ret == NVHDMIPKT_SUCCESS) && b2Heads1Or) {
+        /*
+         * 2Heads1Or requires either YUV420 or DSC; pConfig->dscInfo.bEnableDSC
+         * is assigned by NvHdmi_QueryFRLConfig().
+         */
+        nvAssert(pConfig->dscInfo.bEnableDSC ||
+                    (pHwTimings->yuv420Mode == NV_YUV420_MODE_HW));
+    }
+
+    if (ret == NVHDMIPKT_SUCCESS && (pDscInfo != NULL)) {
+        const NvU64 hdmiLinkRate = GetHdmiFrlLinkRate(pConfig->frlRate);
+
+        nvAssert((hdmiLinkRate != 0) ||
+                    (pConfig->frlRate == HDMI_FRL_DATA_RATE_NONE));
+
+        nvkms_memset(pDscInfo, 0, sizeof(*pDscInfo));
+
+        if ((pConfig->frlRate != HDMI_FRL_DATA_RATE_NONE) &&
+                pConfig->dscInfo.bEnableDSC &&
+                (hdmiLinkRate != 0)) {
+            pDscInfo->type = NV_DSC_INFO_EVO_TYPE_HDMI;
+            pDscInfo->hdmi.dscMode = b2Heads1Or ?
+                NV_DSC_EVO_MODE_DUAL : NV_DSC_EVO_MODE_SINGLE;
+            pDscInfo->hdmi.bitsPerPixelX16 =
+                pConfig->dscInfo.bitsPerPixelX16;
+            ct_assert(sizeof(pDscInfo->hdmi.pps) ==
+                      sizeof(pConfig->dscInfo.pps));
+            nvkms_memcpy(pDscInfo->hdmi.pps,
+                         pConfig->dscInfo.pps,
+                         sizeof(pDscInfo->hdmi.pps));
+            pDscInfo->hdmi.dscHActiveBytes =
+                pConfig->dscInfo.dscHActiveBytes;
+            pDscInfo->hdmi.dscHActiveTriBytes =
+                pConfig->dscInfo.dscHActiveTriBytes;
+            pDscInfo->hdmi.dscHBlankTriBytes =
+                pConfig->dscInfo.dscHBlankTriBytes;
+            pDscInfo->hdmi.dscTBlankToTTotalRatioX1k =
+                pConfig->dscInfo.dscTBlankToTTotalRatioX1k;
+            pDscInfo->hdmi.hblankMin =
+                NV_UNSIGNED_DIV_CEIL(((pHwTimings->pixelClock * 1000) *
+                                          pConfig->dscInfo.dscHBlankTriBytes),
+                                     (hdmiLinkRate / 6));
+        } else {
+            pDscInfo->type = NV_DSC_INFO_EVO_TYPE_DISABLED;
+        }
+    }
 
     return ret == NVHDMIPKT_SUCCESS;
 }
@@ -2143,7 +2275,7 @@ void nvHdmiFrlSetConfig(NVDispEvoRec *pDispEvo, NvU32 head)
     const NVDevEvoRec *pDevEvo = pDispEvo->pDevEvo;
     NVDispHeadStateEvoRec *pHeadState =
         &pDispEvo->headState[head];
-    HDMI_FRL_CONFIG *pFrlConfig = &pHeadState->timings.hdmiFrlConfig;
+    HDMI_FRL_CONFIG *pFrlConfig = &pHeadState->hdmiFrlConfig;
     NVHDMIPKT_RESULT ret;
     NvU32 retries = 0;
     const NvU32 MAX_RETRIES = 5;
